/******************************************************************
 * $Id: routines.c,v 1.5 2001/04/02 12:40:05 quisar Exp $
 *
 * Copyright (c) 2000 Benjamin `Quisar' Lerman.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions  
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 ******************************************************************/

#include <stdio.h>
#include <netdb.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <strings.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>

#include "routines.h"

int connect_server(char *server, int port)
{
 struct hostent *h;
 struct sockaddr_in name;
 int sock;

 h = gethostbyname (server);
 if ( ! h )
 {
  return (-1);
 }
 if ( ( sock = socket (AF_INET, SOCK_STREAM, 0) ) < 0 )
 {
  return (-1);
 }
 memset (&name, 0, sizeof (name));
 name.sin_family = AF_INET;
 name.sin_port = htons (port);
 memcpy (&name.sin_addr, h->h_addr_list[0], h->h_length);
 if ( connect (sock, (struct sockaddr *) &name, sizeof (name)) < 0 )
 {
  return (-1);
 }
 return sock;
}

int complete_write(int fd, char *str)
{
 int nb,i,err;
 
 nb=strlen(str);
 i=0;
 
 while(i<nb)
 {
  err = write(fd, str+i, nb-i);
  if(err == -1)
  {
   return err;
  }
  i+=err;
 }
 return 0;
}

int nbcomplete_write(int fd, char *str, int nb)
{
 int i,err;

 i=0;

 while(i<nb)
 {
  err = write(fd, str+i, nb-i);
  if(err == -1)
  {
   return err;
  }
  i+=err;
 }
 return 0;
}

int fcomplete_write(FILE *fd, char *str)
{
 int nb,i,err;
 
 nb=strlen(str);
 i=0;
 
 while(i<nb)
 {
  err = fwrite(str+i, sizeof(char), nb-i, fd);
  if(err == -1)
  {
   return err;
  }
  i+=err;
 }
 return 0;
}

/* Read a single line from fd. Return 1 if success, 0 if success and end
 * of file and -1 if errors of memory allocation */
int read_single_line(int fd, char **buf)
{
 char *res,*old;
 int i,size,nbread;
 
 i=0;
 size=80;
 res = (char *) malloc(size);
 
 if(res == NULL)
  return -1;
 
 do
 {
  if(i==size)
  {
   size*=2;
   old = res;
   res = (char *) realloc(old, size);
   if(res == NULL) {
    free(old);
    return -1;
   }
  }
  nbread = read(fd, res+i, 1);
  if(nbread)
   i++;
  else
  {
   res[i]='\0';
   res = (char *) realloc(res, i+1);
   *buf = res;
   return 0;
  } 
 } while(res[i-1] != '\n');

 old = res;
 res = (char *) realloc(old, i+1);
 if(res == NULL) {
  free(old);
  return -1;
 }

 res[i] = '\0';
 
 *buf = res;
 return 1;
}

/* Read a single line from fd. Return 1 if success, 0 if end of file
 * and -1 if errors of memory allocation */
int fread_single_line(FILE *fd, char **buf)
{
 char *res,*val;
 int i,size;

 size=81;
 i=0;
 res = (char *) malloc(size * sizeof(char));

 if(res == NULL)
  return -1;

 while(1)
 {
  res[size - 1] = '0';
  val = fgets(res+i, 81, fd);
  if(val == NULL)
  {
   free(res);
   *buf = NULL;
   return 0;
  }
  if(res[size - 1] == '0')
  {
   res = realloc(res, sizeof(char)*(strlen(res) + 1));
   *buf = res;
   return 1;
  }
  size+=80;
  i+=80;
  res = realloc(res, sizeof(char)*size);
 }
}

char **generate_argv(char *commande)
{
 char **res, **tmp_res;
 int nb = 5;
 int index = 0;
 char *p1, *p2;
 int i;
 
 res = (char **) malloc(nb * sizeof(char *));
 if(res == NULL)
  return NULL;

 p1 = commande;
 while(*p1 && ((*p1 == ' ') || (*p1 == '\t'))) p1++;
 while(*p1)
 {
  p2 = p1;
  while(*p2 && (*p2 != ' ') && (*p2 != '\t')) p2++;

  /* Here find the word that interest us. */
  if(index == nb)
  {
   nb *= 2;
   tmp_res = (char **) realloc(res, nb * sizeof(char *));
   if(tmp_res)
    res = tmp_res;
   else
   {
    for(i=0; i<index; i++)
     free(res[i]);
    free(res);
    return NULL;
   }
  }
  res[index] = (char *) malloc(sizeof(char) * (p2 - p1 + 1));
  if(res[index] == NULL)
  {
   for(i=0; i<index; i++)
    free(res[i]);
   free(res);
   return NULL;
  }

  strncpy(res[index], p1, p2 - p1);
  res[index][p2 - p1] = '\0';
  
  index++;
  
  p1 = p2;
  while(*p1 && ((*p1 == ' ') || (*p1 == '\t'))) p1++;
 }
 if(index)
 {
  tmp_res = (char **) realloc(res, (index + 1) * sizeof(char *));
  if(tmp_res)
  {
   res = tmp_res;
   res[index] = NULL;
   return res;
  }
  else
  {
   for(i=0; i<index; i++)
    free(res[i]);
   free(res);
   return NULL;
  }
 }
 else
 {
  free(res);
  return NULL;
 }
}
