/******************************************************************
 * $Id: tbuffy.c,v 1.47 2001/04/02 12:40:05 quisar Exp $
 *
 * Copyright (c) 2000 Benjamin `Quisar' Lerman.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions  
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *   
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 ******************************************************************/

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/poll.h>
#include <sys/wait.h>
#include <unistd.h>
#include <ncurses.h>
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <signal.h>
#include <errno.h>

#include "tbuffy.h"
#include "routines.h"

mboxes_t mboxes[NBMBOXES];  // I keep my mailboxes to watch in this tab.
pipe_t pipes[NBMBOXES];  // I keep the external programs I have to call here.
struct pollfd pollpipe[NBMBOXES];
int nbbox=-1; // The numbers if box I have to watch.
int nbpipe=0; // The numbers of pipes.
int nb_col=1; // The numver of colonnes.

char *name; // The name I've been called with.

char *rcfile=""; // My config file.
char *default_nntpserver=""; // The default newsserver.
char *default_mailserver=""; // The default mailserver.
char *newsrc="";     // The newsrc default file.
char *default_action="";     // The default action
char *default_login="";
char *default_pass="";
int auto_exe = 0;
time_t default_timestamp= (time_t) 10;  // The default waiting beetween to checking

int gere_screen(void); // Here is the main loop.
int initialize(void); // My initialisation function.
int update_mbox(int nb); // How much message are to be read in the box n nb.
int check_mboxes(int i); // Has the ith mailbox changed.
int execute_commande(int cursor); // We hit enter?
int get_news_line(char *group, char *my_newsrc, char *res); // We need to find the line to send to the Perl analizer.
int gere_variable(char **buf, char *box); // We search buf for the $m and $c variables.
int get_new_nbmessage(int fd);

void redraw_screen(char **data, int cursor, int *debut); // We redraw the screen
void init_curses(void); // Initialisation of the ncurses library.
void init_pipes(void); // Initialisation of the pipes.
void quit_curses(void); // We qui ncurses.
void find_rcfile(void); // Find the diffrent default options.

char **generate_command(char *str); // Generate the array for a command

int maxsignal;

int main(int argc, char **argv, char **arge)
{
 int c,i;
 int res;
 char *p1;

#ifdef BSD_COMP
 maxsignal = _NSIG;
#else
 maxsignal = SIGRTMIN;
#endif

 name=argv[0];
 p1=name;
 while(*p1)
 {
  if((*p1 == '/') && (p1[1]))
  {
   name=(p1+1);
  }
  p1++;
 }

 while((c=getopt(argc, argv, "a:f:m:l:p:n:s:t:ehv")) != -1)
 {
  if((c == '?') || (c == 'h'))
  {
   printf("Usage: %s [-v] [-h] [-e] [-f rcfile] [-s newsserver] [-m mailserver] [-n newsrc] [-l login] [-p password] [-t time] [-a action]\n", name);
   printf("  where:\n");
   printf("    file is a configuration file. (default ~/.tbuffyrc or /etc/tbuffyrc)\n");
   printf("    newsserver is the default NNTP server. (default $NNTPSERVER or news)\n");
   printf("    mailserver is the default mail server for POP, APOP and IMAP. (default $MAILSERVER or pop)\n");
   printf("    newsrc is the default newsrc file. (default $HOME/.newsrc)\n");
   printf("    login is the default login. (default none)\n");
   printf("    password is the default password. (default none)\n");
   printf("    time is the default time beetween 2 checks of the mailbox. (default 10)\n");
   printf("    action is the default action. (default none)\n");
   if(c == '?')
    exit(1);
   exit(0);
  }
  else if(c== 'v')
  {
   printf("Tbuffy version %d.%d par Benjamin Lerman, %s.\n", VER_MAJ, VER_MIN, VER_DATE);
   exit(0);
  }
  else if(c == 't')
  {
   default_timestamp = atoi(optarg);
   if(default_timestamp <= 0)
   {
    fprintf(stderr, "%s: %d is not a right timestamp.\n", name, default_timestamp);
    exit(255);
   }
  }
  else if(c == 'e')
   auto_exe = 1;
  else if(c == 'f')
   rcfile = strdup(optarg);
  else if(c == 's')
   default_nntpserver = strdup(optarg);
  else if(c == 'm')
   default_mailserver = strdup(optarg);
  else if(c == 'n')
   newsrc = strdup(optarg);
  else if(c == 'a')
   default_action = strdup(optarg);
  else if(c == 'l')
   default_login = strdup(optarg);
  else if(c == 'p')
   default_pass = strdup(optarg);
 }

 find_rcfile(); 
 
 if(initialize() == -1)
 {
//  perror(name);
  fprintf(stderr, "A problem occured while initialising!\n");
  exit(255);
 }

 if(nbbox == 0)
 {
  fprintf(stderr, "There is nothing to watch!\n");
  exit(255);
 }
 
 /* On passe maintenant  la boucle principale */

 res=gere_screen();

 for(i=0; i<nbpipe; i++)
 {
  complete_write(pipes[i].perl_in, "quit\n");
  
  close(pipes[i].perl_in);
  close(pipes[i].perl_out); 
  if(waitpid(pipes[i].pid, NULL, 0) < 0)
  {
   perror(name);
  }
 }
 
 return res;
}

void find_rcfile(void)
{
 char *path;
 char *home;
 char *nntpserver;
 char *mailserver;
 struct stat buf;
 
 home=getenv("HOME");
 nntpserver=getenv("NNTPSERVER");
 mailserver=getenv("MAILSERVER");
 
 if(! *rcfile)
 {
  path = (char *) malloc(sizeof(char)*(sizeof("/.tbuffyrc") + strlen(home)));
  rcfile="/etc/tbuffyrc";
  if(path)
  {
   strcpy(path, home);
   strcat(path, "/.tbuffyrc");
   if((! stat(path, &buf)) && (S_ISREG(buf.st_mode)))
    rcfile=path;
   else
    free(path);
  }
 }
 if(! *newsrc)
 {
  path = (char *) malloc(sizeof(char)*(sizeof("/.newsrc") + strlen(home)));
  newsrc="/dev/null";
  if(path)
  {
   strcpy(path, home);
   strcat(path, "/.newsrc");
   if((! stat(path, &buf)) && (S_ISREG(buf.st_mode)))
    newsrc=path;
   else
    free(path);
  }
 }
 if(! *default_nntpserver)
 {
  if(nntpserver)
   default_nntpserver=nntpserver;
  else
   default_nntpserver="news";
 }
 if(! *default_mailserver)
 {
  if(mailserver)
   default_mailserver=mailserver;
  else
   default_mailserver="pop";
 }
}

int gere_screen(void)
{
 char **lines_to_draw;
 char *shell;
 int cursor=0, debut=0;
 int i,j;
 int to_redraw=0;
 int c, cont,pid,recheck;
 time_t temp_time;
 fd_set pipeset;
 struct timeval tv;
 struct sigaction sigact;
 
 sigemptyset(&sigact.sa_mask);
 sigact.sa_sigaction = NULL;
 sigact.sa_flags=0;
 init_curses();

 lines_to_draw = (char **) malloc(nbbox * (sizeof (char *)));
 if(lines_to_draw == NULL)
 {
  quit_curses();
  return 1;
 }
 for(i=0; i<nbbox; i++)
 {
  lines_to_draw[i] = (char *) malloc((COLS+1)*sizeof(char));
  if(lines_to_draw[i] == NULL)
  {
   quit_curses();
   return 1;
  }
  for(j=0; j<COLS; j++)
  {
   lines_to_draw[i][j]=' ';
  }
  lines_to_draw[i][COLS]='\0';
  j = strlen(mboxes[i].nom);
  if(j>COLS-4)
  {
   j=COLS-4;
  }
  strncpy(lines_to_draw[i], mboxes[i].nom, j);
 }

 redraw_screen(lines_to_draw, cursor, &debut);
 
 shell = getenv("SHELL");
 if(!shell)
  shell = "/bin/sh";

 recheck = 0; 
 cont = 1;
 
 while(cont)
 {
  c = ERR;
  while(c == ERR)
  {
   temp_time = time(NULL);
   for(i=0; i<nbbox ; i++)
   {
    if(recheck || (temp_time - mboxes[i].last_check > mboxes[i].timestamp))
    {
     check_mboxes(i);
     mboxes[i].last_check = temp_time;
    }
   }
   recheck = 0;
#if 1
   FD_ZERO(&pipeset);
   tv.tv_sec=0;
   tv.tv_usec=25;
   j=1;
   for(i=0; i<nbpipe; i++)
   {
    FD_SET(pipes[i].perl_out, &pipeset);
    if(pipes[i].perl_out >= j)
     j = pipes[i].perl_out + 1;
   }
   if(select(j, &pipeset, NULL, NULL, &tv))
   {
    for(i=0; i<nbpipe; i++)
    {
     if(FD_ISSET(pipes[i].perl_out, &pipeset))
     {
      switch(get_new_nbmessage(pipes[i].perl_out))
      {
       case 0:
	break;
       case 2:
	recheck=1;
       case 1:
	to_redraw=1;
      }
     }
    }
   }
#else
   if(poll(pollpipe, nbpipe, 50))
   {
    for(i=0; i<nbpipe; i++)
    {
     if(pollpipe[i].revents & (POLLIN | POLLPRI))
     {
      to_redraw = get_new_nbmessage(pipes[i].perl_out) || to_redraw;
     }
    }
   }
#endif
   if(to_redraw)
   {
    to_redraw = 0;
    redraw_screen(lines_to_draw, cursor, &debut);
   }
   c = getch();
  }
  switch(c)
  {
   case 'q':
   case 'Q':
    cont = 0;
    break;
   case 0xc:
    clear();
    break;
   case '!':
    quit_curses();
    pid = fork();
    if(pid == -1)
    {
     perror(name);
     return -1;
    }
    if(pid == 0)
    {
     /* Je suis le fils */
     for(i=0; i<nbpipe;i++)
     {
      close(pipes[i].perl_in);
      close(pipes[i].perl_out);
     }
     if(execlp(shell, shell, NULL) == -1)
     {
      perror(name);
      return -1;
     }
    }
    sigact.sa_handler = SIG_IGN;
    for(i=0; i<maxsignal;i++)
     if(i != SIGCHLD)
     {
      sigaction(i, &sigact, NULL);
//      signal(i, SIG_IGN);
     }
    if(waitpid(pid, NULL, 0) < 0)
    {
     perror(name);
     return -1;
    }
    sigact.sa_handler = SIG_DFL;
    for(i=0; i<maxsignal;i++)
     if(i != SIGINT)
     {
      sigaction(i, &sigact, NULL);
//      signal(i, SIG_DFL);
     }
    init_curses();
    recheck=1;
    break;
   case KEY_UP:
   case 'k':
    if(cursor)
     cursor--;
    else
     cursor=nbbox-1;
    break;
   case KEY_DOWN:
   case 'j':
    if(cursor < nbbox-1)
     cursor++;
    else
     cursor=0;
    break;
   case KEY_HOME:
   case KEY_FIND:
    cursor=0;
    break;
   case KEY_END:
   case KEY_SELECT:
    cursor=nbbox-1;
    break;
   case KEY_PPAGE:
    cursor-=LINES;
    if(cursor<0)
     cursor=0;
    break;
   case KEY_NPAGE:
    cursor+=LINES;
    if(cursor > nbbox -1)
     cursor = nbbox-1;
    break;
   case KEY_ENTER:
   case '\n':
   case '\r':
    execute_commande(cursor);
    recheck=1;
    break;
   case '\t':
    i=(cursor+1) %nbbox;
    while((mboxes[i].nbmessages == 0) && (i!=cursor))
    {
     i=(i+1) %nbbox;
    }
    cursor = i;
    break;
   case '`':
   case 178:
    i = cursor - 1;
    if(i<0) i=nbbox-1;
    while((mboxes[i].nbmessages == 0) && (i!=cursor))
    {
     if(i) i--;
     else i=nbbox-1;
    }
    cursor = i;
    break;
   default:
    beep();
    break;
  }
  
  to_redraw=1;
 }
 quit_curses();
 return 0;
}

int get_new_nbmessage(int fd)
{
 int res;
 char *buf;
 int box, nb;

 res = read_single_line(fd, &buf);
 if(res >= 0)
 {
  if(sscanf(buf, "%d %d", &box, &nb) == 2)
  {
   if((nb >= 0) && (box >=0))
   {
    if(nb != mboxes[box].nbmessages)
    {
     if((nb > mboxes[box].nbmessages) && (mboxes[box].nbmessages >= 0))
     {
      beep();
      if(mboxes[box].execute_if_new)
      {
       mboxes[box].nbmessages = nb;
       execute_commande(box);
       return 2;
      }
     }
     mboxes[box].nbmessages = nb;
     return 1;
    }
    return 0;
   }
  }
 }
 return -1;
}

int check_mboxes(int i)
{
 int res;
 unsigned int diff;
 struct stat tmp_stat;
 
 if(mboxes[i].type == MAILBOX)
 {
  stat(mboxes[i].path, &tmp_stat);
  diff = ((unsigned int) tmp_stat.st_size) - ((unsigned int) mboxes[i].size);
  if(diff)
  {
   res = update_mbox(i);
  }
 }
 else if((mboxes[i].type == MDIRBOX) || (mboxes[i].type == POP3BOX) || (mboxes[i].type == APOPBOX) || (mboxes[i].type == IMAPBOX) || (mboxes[i].type == NNTPBOX))
 {
  res = update_mbox(i);
 }
 return res;
}

int initialize(void)
{
 int tbuffyrc;
 int res,i;
 char *buf, *buf2, *buf3, *buf4;
 struct sigaction sigact;
 
 tbuffyrc = open(rcfile, O_RDONLY);
 if(tbuffyrc == -1)
 {
  perror(rcfile);
  return -1;
 }

 res=1;
 while((nbbox<NBMBOXES) && (res == 1))
 { 
  res = read_single_line(tbuffyrc, &buf);
  if(res == 1)
  {
   if(! CSTCMP(buf, "default_command "))
   {
    buf2=(buf+strlen("default_command "));
    default_action = strdup(buf2);
   }
   if(! CSTCMP(buf, "default-auto-execute "))
   {
    buf2=(buf+strlen("default-auto-execute "));
    auto_exe = (buf2[0] == '1');
   }
   if(! CSTCMP(buf, "default_nntpserver "))
   {
    buf2=(buf+strlen("default_nntpserver "));
    buf3 = strtok(buf2, " \t\n");
    if(buf3)
    {
     default_nntpserver = strdup(buf3);
    }
   }
   if(! CSTCMP(buf, "default_pass "))
   {
    buf2=(buf+strlen("default_pass "));
    buf3 = strtok(buf2, " \t\n");
    if(buf3)
    {
     default_pass = strdup(buf3);
    }
   }
   if(! CSTCMP(buf, "default_timestamp "))
   {
    buf2=(buf+strlen("default_timestamp "));
    buf3 = strtok(buf2, " \t\n");
    if(buf3)
    {
     i = atoi(buf3);
     if(i>0)
      default_timestamp = i;
    }
   }
   if(! CSTCMP(buf, "default_login "))
   {
    buf2=(buf+strlen("default_login "));
    buf3 = strtok(buf2, " \t\n");
    if(buf3)
    {
     default_login = strdup(buf3);
    }
   }
   if(! CSTCMP(buf, "default_newsrc "))
   {
    buf2=(buf+strlen("default_newsrc "));
    buf3 = strtok(buf2, " \t\n");
    if(buf3)
    {
     newsrc = strdup(buf3);
    }
   }
   if((nbbox<NBMBOXES-1) && (! CSTCMP(buf, "box ")))
   {
    gere_variable(&buf, "");
    buf2=(buf+strlen("box " ));
    buf3 = strtok(buf2, " \t\n");
    if(buf3)
    {
     nbbox++;
     mboxes[nbbox].type = MAILBOX;
     mboxes[nbbox].execute_if_new = auto_exe;
     mboxes[nbbox].path = strdup(buf3);
     mboxes[nbbox].nom = mboxes[nbbox].path;
     mboxes[nbbox].nntpserver = NULL;
     mboxes[nbbox].mailserver = NULL;
     mboxes[nbbox].newsrc = NULL;
     mboxes[nbbox].login = NULL;
     mboxes[nbbox].pass = NULL;
     mboxes[nbbox].pipe=NULL;
     mboxes[nbbox].timestamp = default_timestamp;
     buf4 = strdup(default_action);
     gere_variable(&buf4, mboxes[nbbox].path);
     mboxes[nbbox].action=generate_command(buf4);
     free(buf4);
     mboxes[nbbox].nbmessages=-1;
     mboxes[nbbox].last_check=time(NULL);
    }
   }
   if((nbbox<NBMBOXES-1) && (! CSTCMP(buf, "imapbox ")))
   {
    gere_variable(&buf, "");
    buf2=(buf+strlen("imapbox " ));
    buf3 = strtok(buf2, " \t\n");
    if(buf3)
    {
     nbbox++;
     mboxes[nbbox].type = IMAPBOX;
     mboxes[nbbox].execute_if_new = auto_exe;
     mboxes[nbbox].path = strdup(buf3);
     mboxes[nbbox].nom = mboxes[nbbox].path;
     mboxes[nbbox].nntpserver = NULL;
     buf4 = strdup(default_mailserver);
     gere_variable(&buf4, mboxes[nbbox].path);
     mboxes[nbbox].mailserver = buf4;
     mboxes[nbbox].newsrc = NULL;
     buf4 = strdup(default_login);
     gere_variable(&buf4, mboxes[nbbox].path);
     mboxes[nbbox].login = buf4;
     buf4 = strdup(default_pass);
     gere_variable(&buf4, mboxes[nbbox].path);
     mboxes[nbbox].pass = buf4;
     mboxes[nbbox].pipe=NULL;
     mboxes[nbbox].timestamp = default_timestamp;
     buf4 = strdup(default_action);
     gere_variable(&buf4, mboxes[nbbox].path);
     mboxes[nbbox].action=generate_command(buf4);
     free(buf4);
     mboxes[nbbox].nbmessages=-1;
     mboxes[nbbox].last_check=time(NULL);
    }
   }
   if((nbbox<NBMBOXES-1) && (! CSTCMP(buf, "popbox ")))
   {
    gere_variable(&buf, "");
    buf2=(buf+strlen("popbox " ));
    buf3 = strtok(buf2, " \t\n");
    if(buf3)
    {
     nbbox++;
     mboxes[nbbox].type = POP3BOX;
     mboxes[nbbox].execute_if_new = auto_exe;
     mboxes[nbbox].path = "";
     mboxes[nbbox].nom = strdup(buf3);
     mboxes[nbbox].nntpserver = NULL;
     mboxes[nbbox].mailserver = mboxes[nbbox].nom;
     mboxes[nbbox].newsrc = NULL;
     buf4 = strdup(default_login);
     gere_variable(&buf4, mboxes[nbbox].path);
     mboxes[nbbox].login = buf4;
     buf4 = strdup(default_pass);
     gere_variable(&buf4, mboxes[nbbox].path);
     mboxes[nbbox].pass = buf4;
     mboxes[nbbox].pipe=NULL;
     mboxes[nbbox].timestamp = default_timestamp;
     buf4 = strdup(default_action);
     gere_variable(&buf4, mboxes[nbbox].path);
     mboxes[nbbox].action=generate_command(buf4);
     free(buf4);
     mboxes[nbbox].nbmessages=-1;
     mboxes[nbbox].last_check=time(NULL);
    }
   }
   if((nbbox<NBMBOXES-1) && (! CSTCMP(buf, "newsbox ")))
   {
    gere_variable(&buf, "");
    buf2=(buf+strlen("newsbox " ));
    buf3 = strtok(buf2, " \t\n");
    if(buf3)
    {
     nbbox++;
     mboxes[nbbox].type = NNTPBOX;
     mboxes[nbbox].execute_if_new = auto_exe;
     mboxes[nbbox].path = strdup(buf3);
     mboxes[nbbox].nom = mboxes[nbbox].path;
     buf4 = strdup(default_nntpserver);
     gere_variable(&buf4, mboxes[nbbox].path);
     mboxes[nbbox].nntpserver = buf4;
     mboxes[nbbox].mailserver = NULL;
     buf4 = strdup(newsrc);
     gere_variable(&buf4, mboxes[nbbox].path);
     mboxes[nbbox].newsrc = buf4;
     buf4 = strdup(default_login);
     gere_variable(&buf4, mboxes[nbbox].path);
     mboxes[nbbox].login = buf4;
     buf4 = strdup(default_pass);
     gere_variable(&buf4, mboxes[nbbox].path);
     mboxes[nbbox].pass = buf4;
     mboxes[nbbox].pipe=NULL;
     mboxes[nbbox].timestamp = default_timestamp;
     buf4 = strdup(default_action);
     gere_variable(&buf4, mboxes[nbbox].path);
     mboxes[nbbox].action=generate_command(buf4);
     free(buf4);
     mboxes[nbbox].nbmessages=-1;
     mboxes[nbbox].last_check=time(NULL);
    }
   }
   if((nbbox>=0) && (! CSTCMP(buf, "auto-execute ")))
   {
    buf2=(buf+strlen("auto-execute " ));
    if(buf2[0] == '1') mboxes[nbbox].execute_if_new = 1;
    else mboxes[nbbox].execute_if_new = 0;
   }
   if((nbbox>=0) && (! CSTCMP(buf, "nntpserver ")))
   {
    gere_variable(&buf, mboxes[nbbox].path);
    buf2=(buf+strlen("nntpserver " ));
    buf3 = strtok(buf2, " \t\n");
    if(buf3)
    {
     mboxes[nbbox].nntpserver = strdup(buf3);
    }
   }
   if((nbbox>=0) && (! CSTCMP(buf, "login ")))
   {
    gere_variable(&buf, mboxes[nbbox].path);
    buf2=(buf+strlen("login " ));
    buf3 = strtok(buf2, " \t\n");
    if(buf3)
    {
     mboxes[nbbox].login = strdup(buf3);
    }
   }
   if((nbbox>=0) && (! CSTCMP(buf, "pass ")))
   {
    gere_variable(&buf, mboxes[nbbox].path);
    buf2=(buf+strlen("pass " ));
    buf3 = strtok(buf2, " \t\n");
    if(buf3)
    {
     mboxes[nbbox].pass = strdup(buf3);
    }
   }
   if((nbbox>=0) && (! CSTCMP(buf, "mailserver ")))
   {
    gere_variable(&buf, mboxes[nbbox].path);
    buf2=(buf+strlen("mailserver " ));
    buf3 = strtok(buf2, " \t\n");
    if(buf3)
    {
     mboxes[nbbox].mailserver = strdup(buf3);
    }
   }
   if((nbbox>=0) && (! CSTCMP(buf, "timestamp ")))
   {
    buf2=(buf+strlen("timestamp "));
    buf3 = strtok(buf2, " \t\n");
    if(buf3)
    {
     mboxes[nbbox].timestamp = atoi(buf3);
     if(mboxes[nbbox].timestamp <= 0)
      mboxes[nbbox].timestamp = default_timestamp;
    }
   }
   if((nbbox>=0) && (! CSTCMP(buf, "type ")))
   {
    gere_variable(&buf, mboxes[nbbox].path);
    buf2=(buf+strlen("type "));
    buf3 = strtok(buf2, " \t\n");
    if(buf3)
    {
     if(! strcmp(buf3, "mail"))
     {
      if((mboxes[nbbox].type == MAILBOX) || (mboxes[nbbox].type == MDIRBOX))
       mboxes[nbbox].type = MAILBOX;
     }
     else if(! strcmp(buf3, "maildir"))
     {
      if((mboxes[nbbox].type == MAILBOX) || (mboxes[nbbox].type == MDIRBOX))
       mboxes[nbbox].type = MDIRBOX;
     }
     else if(! strcmp(buf3, "pop"))
     {
      if((mboxes[nbbox].type == POP3BOX) || (mboxes[nbbox].type == APOPBOX))
       mboxes[nbbox].type = POP3BOX;
     }
     else if(! strcmp(buf3, "apop"))
     {
      if((mboxes[nbbox].type == POP3BOX) || (mboxes[nbbox].type == APOPBOX))
       mboxes[nbbox].type = APOPBOX;
     }
    }
   }
   if((nbbox>=0) && (! CSTCMP(buf, "newsrc ")))
   {
    gere_variable(&buf, mboxes[nbbox].path);
    buf2=(buf+strlen("newsrc "));
    buf3 = strtok(buf2, " \t\n");
    if(buf3)
    {
     mboxes[nbbox].newsrc = strdup(buf3);
    }
   }
   if((nbbox>=0) && (! CSTCMP(buf, "title ")))
   {
    gere_variable(&buf, mboxes[nbbox].path);
    buf2=(buf+strlen("title "));
    while(*buf2 == ' ')
    {
     buf2++;
    }
    buf3=buf2;
    while(*buf3 && (*buf3 != '\n'))
    {
     buf3++;
    }
    if(*buf3)
    {
     *buf3='\0';
    }
    mboxes[nbbox].nom = strdup(buf2);
   }
   if((nbbox>=0) && (! CSTCMP(buf, "command ")))
   {
    gere_variable(&buf, mboxes[nbbox].path);
    buf2 = (buf+strlen("command "));
    mboxes[nbbox].action = generate_command(buf2);
   }
   free(buf);
  }
 }

 close(tbuffyrc);

 if(res == -1)
 {
  perror(name);
  return -1;
 }

 nbbox++;

 init_pipes();
 
 for(res=0; res<nbbox; res++)
 {
  if(update_mbox(res) == -1)
  {
   fprintf(stderr, "Error while trying to check %s : %s\n", mboxes[res].path, strerror(errno));
   return -1;
  }
  mboxes[res].last_check = time(NULL);
#if 0

  printf("Path %s\nName :%s\nSize : %u\nNbMessage : %d\n",
    mboxes[res].path,
    mboxes[res].nom,
    (unsigned int) mboxes[res].size,
    mboxes[res].nbmessages);
  if(mboxes[res].action)
  {
   i=0;
   while(mboxes[res].action[i])
   {
    printf("%sa\n", mboxes[res].action[i]);
    i++;
   }
   printf("\n");
  }
  printf("\n");
 
#endif

 }

 sigact.sa_handler = SIG_IGN;
 sigaction(SIGINT, &sigact, NULL);
// signal(SIGINT, SIG_IGN); 
 return (0);
}

int update_mbox(int nb)
{
 struct stat get_stat;
 char numero[10];
 char ask[256];

 sprintf(numero, "%d ", nb);
 
 if(mboxes[nb].type == MAILBOX)
 {
  if(stat(mboxes[nb].path, &get_stat) == -1)
  {
   return -1;
  }
  mboxes[nb].size=get_stat.st_size;
  complete_write(mboxes[nb].pipe->perl_in, numero);
  complete_write(mboxes[nb].pipe->perl_in, mboxes[nb].path);
  complete_write(mboxes[nb].pipe->perl_in, "\n");
  return 1;
 }
 else if (mboxes[nb].type == MDIRBOX)
 {
  complete_write(mboxes[nb].pipe->perl_in, numero);
  complete_write(mboxes[nb].pipe->perl_in, mboxes[nb].path);
  complete_write(mboxes[nb].pipe->perl_in, "\n");
  return 1;
 }
 else if (mboxes[nb].type == POP3BOX)
 {
  complete_write(mboxes[nb].pipe->perl_in, numero);
  complete_write(mboxes[nb].pipe->perl_in, "pop3 ");
  complete_write(mboxes[nb].pipe->perl_in, mboxes[nb].mailserver);
  complete_write(mboxes[nb].pipe->perl_in, " ");
  complete_write(mboxes[nb].pipe->perl_in, mboxes[nb].login);
  complete_write(mboxes[nb].pipe->perl_in, " ");
  complete_write(mboxes[nb].pipe->perl_in, mboxes[nb].pass);
  complete_write(mboxes[nb].pipe->perl_in, "\n");
  return 1;
 }
 else if (mboxes[nb].type == APOPBOX)
 {
  complete_write(mboxes[nb].pipe->perl_in, numero);
  complete_write(mboxes[nb].pipe->perl_in, "apop ");
  complete_write(mboxes[nb].pipe->perl_in, mboxes[nb].mailserver);
  complete_write(mboxes[nb].pipe->perl_in, " ");
  complete_write(mboxes[nb].pipe->perl_in, mboxes[nb].login);
  complete_write(mboxes[nb].pipe->perl_in, " ");
  complete_write(mboxes[nb].pipe->perl_in, mboxes[nb].pass);
  complete_write(mboxes[nb].pipe->perl_in, "\n");
  return 1;
 }
 else if(mboxes[nb].type == IMAPBOX)
 {
  complete_write(mboxes[nb].pipe->perl_in, numero);
  complete_write(mboxes[nb].pipe->perl_in, mboxes[nb].path);
  complete_write(mboxes[nb].pipe->perl_in, "\n");
  return 1;
 }
 else if (mboxes[nb].type == NNTPBOX)
 {
  if(get_news_line(mboxes[nb].path, mboxes[nb].newsrc, ask) == -1)
  {
   mboxes[nb].nbmessages = -1;
  }
  else
  {
   complete_write(mboxes[nb].pipe->perl_in, numero);
   complete_write(mboxes[nb].pipe->perl_in, ask);
   complete_write(mboxes[nb].pipe->perl_in, "\n");
   return 1;
  }
  return 0;
 }
 return 0;
}

void redraw_screen(char **data, int cursor, int *debut)
{
 int i, nbmes,attribut;
 int fin;

 if(cursor < *debut)
  *debut = cursor;
 if(cursor - *debut >= LINES)
  *debut = cursor;
 
 fin = nbbox;
 if(fin - *debut > LINES)
  fin = *debut + LINES;
 
 erase();
 
 for(i=*debut; i<fin; i++)
 {
  nbmes = mboxes[i].nbmessages;
  if(nbmes > 999)
  {
   nbmes=999;
  }
  if(nbmes >= 0) sprintf(data[i] + (COLS - 3), "%3d", nbmes);
  else sprintf(data[i] + (COLS - 3), "  ?");
  attribut = A_NORMAL;
  if(nbmes > 0)
  {
   attribut |= A_BOLD;
  }
  if(i==cursor)
  {
   attribut |= A_REVERSE;
  }
  attrset(attribut);
  addstr(data[i]);
 }
 
 refresh();
}

int execute_commande(int nb)
{
 int pid,i;
 struct sigaction sigact;
 
 sigemptyset(&sigact.sa_mask);
 sigact.sa_sigaction = NULL;
 sigact.sa_flags=0;

 if(mboxes[nb].action)
 {
  quit_curses();
  pid = fork();
  if(pid == -1)
  {
   perror(name);
   return -1;
  }
  if(pid == 0)
  {
   /* Je suis le fils */
   for(i=0; i<nbpipe; i++)
   {
    close(pipes[i].perl_in);
    close(pipes[i].perl_out);
   }
   if(execvp(mboxes[nb].action[0], mboxes[nb].action) == -1)
   {
    perror(name);
    return -1;
   }
  }
  sigact.sa_handler = SIG_IGN;
  for(i=0; i<maxsignal;i++)
   if(i != SIGCHLD)
   {
    sigaction(i, &sigact, NULL);
//    signal(i, SIG_IGN);
   }
  if(waitpid(pid, NULL, 0) < 0)
  {
   perror(name);
   return -1;
  }
  sigact.sa_handler = SIG_DFL;
  for(i=0; i<maxsignal;i++)
   if(i != SIGINT)
   {
    sigaction(i, &sigact, NULL);
//    signal(i, SIG_DFL);
   }
  init_curses();
 }
 return 0;
}

void init_curses(void)
{
 initscr();
 curs_set(0);

 noecho();
 nonl();
 cbreak();
 nodelay(stdscr, TRUE);
 intrflush(stdscr, FALSE);
 keypad(stdscr, TRUE); 
}

void quit_curses(void)
{
 endwin();
}

void init_pipes(void)
{
 int i,j;
 int pipe_fd1[2];
 int pipe_fd2[2];
  
 for(i=0; i<nbbox; i++)
 {
  if(mboxes[i].type == MAILBOX)
  {
   for(j=0; (j<nbpipe) && strcmp(MAILPIPE, pipes[j].prog); j++); 
   mboxes[i].pipe=(pipes + j);
   if(j == nbpipe)
   {
    if(pipe(pipe_fd1) == -1)
    {
     perror(name); 
     exit (255);
    }
    if(pipe(pipe_fd2) == -1)
    {
     perror(name);
     exit (255);
    }
    pipes[nbpipe].pid = fork();

    if(pipes[nbpipe].pid == -1)
    {
     perror(name);
     exit(255);
    }
    if(pipes[nbpipe].pid == 0) /* I'm the son */
    {
     dup2(pipe_fd1[0], 0);
     dup2(pipe_fd2[1], 1);
     dup2(pipe_fd2[1], 2);
     close(pipe_fd1[0]);
     close(pipe_fd1[1]);
     close(pipe_fd2[0]);
     close(pipe_fd2[1]);
     execlp(MAILPIPE, MAILPIPE, NULL);
    }
    close(pipe_fd1[0]);
    close(pipe_fd2[1]);
    pipes[nbpipe].perl_out = pipe_fd2[0];
    pipes[nbpipe].perl_in = pipe_fd1[1];
    pipes[nbpipe].prog = MAILPIPE;
    pipes[nbpipe].arg = NULL;
    nbpipe++;
   }
  }
  else if(mboxes[i].type == MDIRBOX)
  {
   for(j=0; (j<nbpipe) && strcmp(MAILDIRPIPE, pipes[j].prog); j++); 
   mboxes[i].pipe=(pipes + j);
   if(j == nbpipe)
   {
    if(pipe(pipe_fd1) == -1)
    {
     perror(name); 
     exit (255);
    }
    if(pipe(pipe_fd2) == -1)
    {
     perror(name);
     exit (255);
    }
    pipes[nbpipe].pid = fork();

    if(pipes[nbpipe].pid == -1)
    {
     perror(name);
     exit(255);
    }
    if(pipes[nbpipe].pid == 0) /* I'm the son */
    {
     dup2(pipe_fd1[0], 0);
     dup2(pipe_fd2[1], 1);
     dup2(pipe_fd2[1], 2);
     close(pipe_fd1[0]);
     close(pipe_fd1[1]);
     close(pipe_fd2[0]);
     close(pipe_fd2[1]);
     execlp(MAILDIRPIPE, MAILDIRPIPE, NULL);
    }
    close(pipe_fd1[0]);
    close(pipe_fd2[1]);
    pipes[nbpipe].perl_out = pipe_fd2[0];
    pipes[nbpipe].perl_in = pipe_fd1[1];
    pipes[nbpipe].prog = MAILDIRPIPE;
    pipes[nbpipe].arg = NULL;
    nbpipe++;
   }
  }
  else if((mboxes[i].type == POP3BOX) || (mboxes[i].type == APOPBOX))
  {
   for(j=0; (j<nbpipe) && strcmp(MAILPOPPIPE, pipes[j].prog); j++); 
   mboxes[i].pipe=(pipes + j);
   if(j == nbpipe)
   {
    if(pipe(pipe_fd1) == -1)
    {
     perror(name); 
     exit (255);
    }
    if(pipe(pipe_fd2) == -1)
    {
     perror(name);
     exit (255);
    }
    pipes[nbpipe].pid = fork();

    if(pipes[nbpipe].pid == -1)
    {
     perror(name);
     exit(255);
    }
    if(pipes[nbpipe].pid == 0) /* I'm the son */
    {
     dup2(pipe_fd1[0], 0);
     dup2(pipe_fd2[1], 1);
     dup2(pipe_fd2[1], 2);
     close(pipe_fd1[0]);
     close(pipe_fd1[1]);
     close(pipe_fd2[0]);
     close(pipe_fd2[1]);
     execlp(MAILPOPPIPE, MAILPOPPIPE, NULL);
    }
    close(pipe_fd1[0]);
    close(pipe_fd2[1]);
    pipes[nbpipe].perl_out = pipe_fd2[0];
    pipes[nbpipe].perl_in = pipe_fd1[1];
    pipes[nbpipe].prog = MAILPOPPIPE;
    pipes[nbpipe].arg = NULL;
    nbpipe++;
   }
  }
  else if(mboxes[i].type == IMAPBOX)
  {
   char *arg;
   int size;
   size = 2 + strlen(mboxes[i].mailserver) + strlen(mboxes[i].login);
   arg = (char *) malloc(sizeof(char) * size);
   if(arg == NULL)
   {
    perror(name);
    exit(255);
   }
   sprintf(arg, "%s %s", mboxes[i].mailserver, mboxes[i].login);
   for(j=0; (j<nbpipe) && (strcmp(MAILIMAPPIPE, pipes[j].prog) || strcmp(arg, pipes[j].arg)); j++); 
   mboxes[i].pipe=(pipes + j);
   if(j == nbpipe)
   {
    if(pipe(pipe_fd1) == -1)
    {
     perror(name); 
     exit (255);
    }
    if(pipe(pipe_fd2) == -1)
    {
     perror(name);
     exit (255);
    }
    pipes[nbpipe].pid = fork();

    if(pipes[nbpipe].pid == -1)
    {
     perror(name);
     exit(255);
    }
    if(pipes[nbpipe].pid == 0) /* I'm the son */
    {
     dup2(pipe_fd1[0], 0);
     dup2(pipe_fd2[1], 1);
     dup2(pipe_fd2[1], 2);
     close(pipe_fd1[0]);
     close(pipe_fd1[1]);
     close(pipe_fd2[0]);
     close(pipe_fd2[1]);
     execlp(MAILIMAPPIPE, MAILIMAPPIPE, mboxes[i].mailserver, NULL);
    }
    close(pipe_fd1[0]);
    close(pipe_fd2[1]);
    pipes[nbpipe].perl_out = pipe_fd2[0];
    pipes[nbpipe].perl_in = pipe_fd1[1];
    pipes[nbpipe].prog = MAILIMAPPIPE;
    pipes[nbpipe].arg = arg;
    complete_write(pipes[nbpipe].perl_in, mboxes[i].login);
    complete_write(pipes[nbpipe].perl_in, " ");
    complete_write(pipes[nbpipe].perl_in, mboxes[i].pass);
    complete_write(pipes[nbpipe].perl_in, "\n");
    nbpipe++;
   }
  }
  else if(mboxes[i].type == NNTPBOX)
  {
   for(j=0; (j<nbpipe) && (strcmp(NEWSPIPE, pipes[j].prog) || strcmp(mboxes[i].nntpserver, pipes[j].arg)); j++); 
   mboxes[i].pipe=(pipes + j);
   if(j == nbpipe)
   {
    if(pipe(pipe_fd1) == -1)
    {
     perror(name); 
     exit (255);
    }
    if(pipe(pipe_fd2) == -1)
    {
     perror(name);
     exit (255);
    }
    pipes[nbpipe].pid = fork();

    if(pipes[nbpipe].pid == -1)
    {
     perror(name);
     exit(255);
    }
    if(pipes[nbpipe].pid == 0) /* I'm the son */
    {
     dup2(pipe_fd1[0], 0);
     dup2(pipe_fd2[1], 1);
     dup2(pipe_fd2[1], 2);
     close(pipe_fd1[0]);
     close(pipe_fd1[1]);
     close(pipe_fd2[0]);
     close(pipe_fd2[1]);
     execlp(NEWSPIPE, NEWSPIPE, mboxes[i].nntpserver, NULL);
    }
    close(pipe_fd1[0]);
    close(pipe_fd2[1]);
    pipes[nbpipe].perl_out = pipe_fd2[0];
    pipes[nbpipe].perl_in = pipe_fd1[1];
    pipes[nbpipe].prog = NEWSPIPE;
    pipes[nbpipe].arg = mboxes[i].nntpserver;
    complete_write(pipes[nbpipe].perl_in, mboxes[i].login);
    complete_write(pipes[nbpipe].perl_in, " ");
    complete_write(pipes[nbpipe].perl_in, mboxes[i].pass);
    complete_write(pipes[nbpipe].perl_in, "\n");
    nbpipe++;
   }
  }
 }
 for(i=0; i<nbpipe;i++)
 {
  pollpipe[i].fd = pipes[i].perl_out;
  pollpipe[i].events = POLLIN | POLLPRI;
 }
}

int get_news_line(char *group, char *my_newsrc, char *res)
{
 int newsrc_f;
 int res_read, i;
 char *buf;

 strncpy(res, group, 254);
 res[254]='\0';
 strcat(res, ":");
 i = strlen(res);
 
 newsrc_f = open(my_newsrc, O_RDONLY);
 if(newsrc_f != -1)
 {
  res_read = read_single_line(newsrc_f, &buf);
  while(res_read == 1)
  {
   if(!(strncmp(res, buf, i)))
   {
    strncpy(res,buf,255);
    res[255]='\0';
    free(buf);
    for(buf=res; *buf; buf++)
    {
     if(*buf == '\n')
      *buf = '\0';
    }
    
    close(newsrc_f);
    return 0;
   }
   free(buf);
   res_read = read_single_line(newsrc_f, &buf);
  }
  close(newsrc_f);
 }
 return 0;
  
}

int gere_variable(char **buf, char *box)
{
 char *mailbox,*home,*username;
 char *old,*res,*tmp,*tmp2,*tmp3;
 int size, i;
 
 mailbox = getenv("MAIL");
 home = getenv("HOME");
 username = getenv("USERNAME");
 if(mailbox == NULL)
  mailbox = "";
 if(username == NULL)
  username = "";
 size = 80;
 i=0;
 res = (char *) malloc(size * sizeof(char));
 if(res == NULL)
 {
  return 1;
 }
 for(tmp=*buf; *tmp; tmp++)
 {
  if(i==size)
  {
   size*=2;
   old = res;
   res = (char *) realloc(old, size * sizeof(char));
   if(res == NULL)
   {
    free(old);
    return 1;
   }
  }
  if(*tmp == '$')
  {
   tmp++;
   switch (*tmp)
   {
    case 'm' :
    case 'c' :
    case 'h' :
    case 'u' :
     switch (*tmp)
     {
      case 'm' :
       tmp3 = mailbox;
       break;
      case 'c' :
       tmp3 = box;
       break;
      case 'h' :
       tmp3 = home;
       break;
      case 'u' :
       tmp3 = username;
       break;
     }
     
     while(i+strlen(tmp3) > size)
     {
      size*=2;
      old = res;
      res = (char *) realloc(old, size * sizeof(char));
      if(res == NULL)
      {
       free(old);
       return 1;
      }
     }
     for(tmp2 = tmp3; *tmp2; tmp2++)
     {
      res[i] = *tmp2;
      i++;
     }
     break;
    default:
     res[i] = *tmp;
     i++;
     break;
   }
  }
  else
  {
   res[i] = *tmp;
   i++;
  }
 }
 old = res;
 res = (char *)realloc(old, sizeof(char)*(i+1));
 if(res == NULL)
 {
  free(old);
  return 1;
 }
 res[i] = '\0';
 free(*buf);
 *buf = res;
 return 0;
}

char **generate_command(char *str)
{
 char **res;
 char *buf; 
 int i;

 res = (char **) malloc(10 * sizeof(char *));
 if(res == NULL)
 {
  perror(name);
  return NULL;
 }

 buf = strtok(str, " \t\n");
 i=0;
 while((i<9) && (buf !=NULL))
 {
  res[i] = strdup(buf);
  i++;
  buf = strtok(NULL, " \n");
 }
 if(i)
 {
  res[i] = NULL;
  return res;
 }
 else
 {
  free(res);
  return NULL;
 }
}

